#==============================================================================
# 3D Wire DXF Exporter
#
# Copyright, ILHYOUNG KIM.
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Date: 11/15, 2008
# Version 1.01
# Author: ILHYOUNG KIM, based on the sketchup_to_dxf_stl of Nathan Bromham
#         e-mail: ilhyoungkim@gmail.com
#
# History:
# 1.0  - 11/10/2008 : Initial Version
# 1.01 - 11/15/2008 : Fixed to export Group and Component Object.
# 1.02 -  5/13/2009 : Added 420 dxf color group coding (drlava)
#
#==============================================================================
require 'sketchup.rb'

$scriptName = "export_3d_wire_dxf.rb"

def export_face_file ( file_type )
	model = Sketchup.active_model
 	$faces_array = []
 	model.start_operation "export_face_file"
 	
	out_name = UI.openpanel( file_type+" file location", "." , "#{File.basename(model.path).split(".")[0]}." +file_type )
	dxf_file = File.new( out_name , "w" )

	extractFaces( model.entities )
	extract_component( model )
	
	export_dxf dxf_file
	UI.messagebox( "Export is done!" )
	
 	model.commit_operation
end

def extractFaces( entries )
	entries.each do |element|
		if( element.typename == "Face")
			$faces_array << element
		end
	end
end

def extract_component( model )
	comp_def_array = model.definitions	
	
  	comp_def_array.each do |comp_def|
	  	comp_def.instances.each do |comp_inst|
			if( comp_inst.typename == "ComponentInstance")
  				extractFaces( comp_inst.explode )
  			elsif( comp_inst.typename == "Group")
  				extractFaces( comp_inst.copy.explode )
  			end
 		end
	end
end

def export_dxf( dxf_file )
	model = Sketchup.active_model
	model_filename = File.basename(model.path)
  	if( model_filename == "" )
	    model_filename = "no_name"
  	end
  	model_name = model_filename.split(".")[0]
  	dxf_file.puts( " 0")
  	dxf_file.puts( "SECTION")
  	dxf_file.puts( " 2")
  	dxf_file.puts( "ENTITIES")
  	i = 0
  	
#	UI.messagebox( "Total Faces:" + $faces_array.length.to_s )
	UI.messagebox( "Total Unique Faces:" + $faces_array.uniq.length.to_s )
	
  	$faces_array.uniq.each do |face|
    	loops = face.loops
  		loops.each do |oneLoop|
	    	dxf_file.puts( "  0")
	    	dxf_file.puts( "POLYLINE")
	    	dxf_file.puts( "  8")
	    	dxf_file.puts( "DEFAULT")
	    	dxf_file.puts( " 66")
	    	dxf_file.puts( "     1")
	    	dxf_file.puts( " 70")
	    	dxf_file.puts( "     8")
	    	dxf_file.puts( " 10")
	    	dxf_file.puts( "0.0")
	    	dxf_file.puts( " 20")
	    	dxf_file.puts( "0.0")
	    	dxf_file.puts( " 30")
	    	dxf_file.puts( "0.0")
	    	dxf_file.puts( " 420")
	    	c = face.material
	    	if (c)
	    		c = c.color
	    	else
	    		c = face.back_material
	    		if (c)
	    			c = c.color
	    		else
	    			c = Sketchup::Color.new "White"
	    		end
	    	end
	    	dxf_file.puts((c.red * 65536 + c.green * 256 + c.blue).to_s)
	 		vertices = oneLoop.vertices
	  		vertices.each do |vertex|
	      		dxf_file.puts( "  0")
	      		dxf_file.puts( "VERTEX")
	      		dxf_file.puts( "  8")
	      		dxf_file.puts( "DEFAULT")
	      		dxf_file.puts( " 10")
	      		dxf_file.puts(vertex.position.x.to_f.to_s)
	      		dxf_file.puts( " 20")
	      		dxf_file.puts(vertex.position.y.to_f.to_s)
	      		dxf_file.puts( " 30")
	      		dxf_file.puts(vertex.position.z.to_f.to_s)
		    	dxf_file.puts( " 70")
		    	dxf_file.puts( "     32")
	    	end
	    	if (vertices.length > 0)
	      		dxf_file.puts( "  0")
	      		dxf_file.puts( "VERTEX")
	      		dxf_file.puts( "  8")
	      		dxf_file.puts( "DEFAULT")
	      		dxf_file.puts( " 10")
	      		dxf_file.puts(vertices[0].position.x.to_f.to_s)
	      		dxf_file.puts( " 20")
	      		dxf_file.puts(vertices[0].position.y.to_f.to_s)
	      		dxf_file.puts( " 30")
	      		dxf_file.puts(vertices[0].position.z.to_f.to_s)
		    	dxf_file.puts( " 70")
		    	dxf_file.puts( "     32")
	    	end
	    	dxf_file.puts( "  0")
	    	dxf_file.puts( "SEQEND")
	    	i=i+1
  		end    
  	end
  	dxf_file.puts( "  0")
  	dxf_file.puts( "ENDSEC")
  	dxf_file.puts( "  0")
  	dxf_file.puts( "EOF")
  	dxf_file.close
end

if( not file_loaded?($scriptName) )
    # This will add a separator to the menu, but only once
    #Note: We don't translate the Menu names - the Ruby API assumes you are 
    #using English names for Menus.
    add_separator_to_menu("Plugins")
    
	UI.menu("Plugins").add_item("Export DXF(3D-Wire)") { 
		export_face_file ( "dxf" ) 
	}
end
file_loaded($scriptName)



